package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.Objects;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.regex.MatchResult;
import java.util.regex.Pattern;
import xyz.lychee.lagfixer.libs.adventure.builder.AbstractBuilder;
import xyz.lychee.lagfixer.libs.adventure.util.Buildable;
import xyz.lychee.lagfixer.libs.adventure.util.IntFunction2;
import xyz.lychee.lagfixer.libs.annotations.Contract;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.examination.Examinable;
import xyz.lychee.lagfixer.libs.lang.annotations.RegExp;

public interface TextReplacementConfig extends Buildable<TextReplacementConfig, Builder>, Examinable {
   @NotNull
   static @NotNull Builder builder() {
      return new TextReplacementConfigImpl.Builder();
   }

   @NotNull Pattern matchPattern();

   @FunctionalInterface
   public interface Condition {
      @NotNull PatternReplacementResult shouldReplace(final @NotNull MatchResult result, final int matchCount, final int replaced);
   }

   public interface Builder extends AbstractBuilder<TextReplacementConfig>, Buildable.Builder<TextReplacementConfig> {
      @Contract("_ -> this")
      default Builder matchLiteral(final String literal) {
         return this.match(Pattern.compile(literal, 16));
      }

      @Contract("_ -> this")
      @NotNull
      default @NotNull Builder match(@RegExp final @NotNull String pattern) {
         return this.match(Pattern.compile(pattern));
      }

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder match(final @NotNull Pattern pattern);

      @NotNull
      default @NotNull Builder once() {
         return this.times(1);
      }

      @Contract("_ -> this")
      @NotNull
      default @NotNull Builder times(final int times) {
         return this.condition((index, replaced) -> {
            return replaced < times ? PatternReplacementResult.REPLACE : PatternReplacementResult.STOP;
         });
      }

      @Contract("_ -> this")
      @NotNull
      default @NotNull Builder condition(final @NotNull IntFunction2<PatternReplacementResult> condition) {
         return this.condition((result, matchCount, replaced) -> {
            return (PatternReplacementResult)condition.apply(matchCount, replaced);
         });
      }

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder condition(@NotNull final @NotNull Condition condition);

      @Contract("_ -> this")
      @NotNull
      default @NotNull Builder replacement(final @NotNull String replacement) {
         Objects.requireNonNull(replacement, "replacement");
         return this.replacement((builder) -> {
            return builder.content(replacement);
         });
      }

      @Contract("_ -> this")
      @NotNull
      default Builder replacement(@Nullable final ComponentLike replacement) {
         Component baked = ComponentLike.unbox(replacement);
         return this.replacement((result, input) -> {
            return baked;
         });
      }

      @Contract("_ -> this")
      @NotNull
      default @NotNull Builder replacement(final @NotNull Function<TextComponent.Builder, @Nullable ComponentLike> replacement) {
         Objects.requireNonNull(replacement, "replacement");
         return this.replacement((result, input) -> {
            return (ComponentLike)replacement.apply(input);
         });
      }

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder replacement(final @NotNull BiFunction<MatchResult, TextComponent.Builder, @Nullable ComponentLike> replacement);
   }
}
